/* ----------------------------------------------------------------------------
*
* Raima Database Manager
*
* Copyright (c) 2013 Raima Inc.,  All rights reserved.
*
* Use of this software, whether in source code format, or in executable,
* binary object code form, is governed by the Raima LICENSE which
* is fully described in the LICENSE.TXT file, included within this
* distribution of files.
*
* ----------------------------------------------------------------------------
*/

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* CursorBulkInsert_db.h was generated from the DDL file CursorBulkInsert_db.ddl. 
* See subsequent examples for DDL modification.
*/
#include "CursorBulkInsert_db.h"

/* CursorBulkInsert_db_dbd.h was generated from the DDL file CursorBulkInsert_db.ddl by the 
* ddlp utility. It is required to use the d_open_ptr API.
*/
#include "CursorBulkInsert_db_dbd.h"

/* EXAMPLE - Hello World with Bulk Insert using Cursors */
/* Each time you run this example, 3 new records are created in the
* database, then all existing records are read from the database
* and printed.
*
* The DDL (Database Definition Language) is in CursorBulkInsert_db.ddl. The database
* has been pre-built and initialized.
*
* The following RDM Functions are illustrated:
*      d_opentask/d_closetask
*      d_open/d_iclose
*      d_cursorAlloc/dc_cursorFree
*      d_trbegin/d_trend
*      d_fillnewArray
*      dc_getFields
*      dc_first/dc_next
*      d_recread
* They are fully documented in http://www.raima.com/support/online-documentation/.
*
* Each of these functions returns an integer status code, where the
* value S_OKAY indicates a successful call.
*
* For simplicity, this example does not check all return codes, but good
* programming practices would dictate that they are checked after each
* RDM call.
*
* The actual database is stored in a directory named 'CursorBulkInsert_db' in this
* project directory.
*/

int32_t main (void) 
{
    RDM_TASK task;
    int32_t iStatus;

    /* Create the context for the database operations */
    iStatus = d_opentask (&task);
    if (iStatus == S_OKAY)
    {
        /* Open the database for exclusive access */
        iStatus = d_open_ptr ("CursorBulkInsert_db", "x", &CursorBulkInsert_db_dbd, sizeof (CursorBulkInsert_db_dbd), task);

        if (iStatus == S_OKAY)
        {
            struct HelloRecord Hello_input[3];
            RDM_RAWSTAT rStatArray[3];
            RDM_CURSOR cursor = NULL;

            iStatus = d_initialize (task, CURR_DB);

            if (iStatus == S_OKAY)
                iStatus = d_cursorAlloc (&cursor, task, 0);

            /* Create 3 records in the new database */
            strncpy (Hello_input[0].message, "Hello World1!", SIZEOF_MESSAGE);
            strncpy (Hello_input[1].message, "Hello World2!", SIZEOF_MESSAGE);
            strncpy (Hello_input[2].message, "Hello World3!", SIZEOF_MESSAGE);

            if (iStatus == S_OKAY)
            {
                /* Start a Insert database transaction. */
                iStatus = d_trbegin ("Insert", task);
                if (iStatus == S_OKAY)
                {
                    iStatus = d_reclock (HELLORECORD, "w", task, 0);

                    /* Create multiple records in the new database */
                    if (iStatus == S_OKAY)
                        d_fillnewArray (HELLORECORD, Hello_input, sizeof (struct HelloRecord), RLEN (Hello_input), rStatArray, task, dc_getDBN (cursor));

                    iStatus = d_trend (task);
                } 

                /* Start Retrieving data from the Database. */
                if (iStatus == S_OKAY)
                {
                    /* Scan and read all created records.
                    * Three additional records will be created each time you run this example.
                    */
                    iStatus = d_getRecords (HELLORECORD, cursor, task,
                        dc_getDBN (cursor));
                }
                if (iStatus == S_OKAY)
                    iStatus = dc_first (cursor);

                while (iStatus == S_OKAY)
                {
                    struct HelloRecord recS;
                    iStatus = dc_getFields (cursor, HELLORECORD, &recS,
                        sizeof(recS), NULL);
                    if (iStatus == S_OKAY)
                    {
                        printf ("%s\n", recS.message);
                        iStatus = dc_next (cursor);
                    }
                }

                iStatus = dc_cursorFree (cursor);
            }
            /* Close the database */
            if (iStatus == S_OKAY)
                iStatus = d_iclose (task, CURR_DB);
            else
                d_iclose (task, CURR_DB);
        }

        if (iStatus == S_OKAY)
            iStatus = d_closetask (task);
        else
            d_closetask (task);
    }

    printf ("Press <Enter> to continue . . .\n");
    getchar ();
}
